/*************************************************
* BigInt Addition/Subtraction Source File        *
* (C) 1999-2003 The Botan Project                *
*************************************************/

#include <botan/mp_core.h>
#include <botan/mp_mul.h>
#include <string.h>

extern "C" {

/*************************************************
* Two Operand Addition                           *
*************************************************/
void bigint_add2(word* const x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   word carry = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      dword sum = ((dword)x[j] + y[j]) + carry;
      x[j] = MP_LOW_WORD(sum);
      carry = MP_HIGH_WORD(sum);
      }

   if(!carry) return;

   for(u32bit j = y_size; j != x_size; j++)
      {
      x[j]++;
      if(x[j]) return;
      }
   x[x_size]++;
   }

/*************************************************
* Three Operand Addition                         *
*************************************************/
void bigint_add3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   if(x_size < y_size)
      {
      const word* t_ptr = x; x = y; y = t_ptr;
      u32bit t_size = x_size; x_size = y_size; y_size = t_size;
      }

   word carry = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      dword sum = ((dword)x[j] + y[j]) + carry;
      z[j] = MP_LOW_WORD(sum);
      carry = MP_HIGH_WORD(sum);
      }

   for(u32bit j = y_size; j != x_size; j++)
      {
      if(carry == 0)
         {
         memcpy(z + j, x + j, (x_size - j) * sizeof(word));
         break;
         }
      z[j] = x[j] + carry;
      carry = ((x[j] == MP_WORD_MAX) ? 1 : 0);
      }
   z[x_size] = carry;
   }

/*************************************************
* Two Operand Subtraction                        *
*************************************************/
void bigint_sub2(word* const x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   const dword MP_RADIX = (dword)1 << MP_WORD_BITS;

   word borrow = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      dword temp = ((MP_RADIX + x[j]) - y[j]) - borrow;
      borrow = ((temp < MP_RADIX) ? 1 : 0);
      x[j] = (word)(temp - MP_RADIX);
      }
   for(u32bit j = y_size; j != x_size; j++)
      {
      if(borrow == 0)
         break;
      dword temp = (MP_RADIX + x[j]) - borrow;
      borrow = ((temp < MP_RADIX) ? 1 : 0);
      x[j] = (word)(temp - MP_RADIX);
      }
   }

/*************************************************
* Three Operand Subtraction                      *
*************************************************/
void bigint_sub3(word* const z,
                 const word* x, u32bit x_size,
                 const word* y, u32bit y_size)
   {
   const dword MP_RADIX = (dword)1 << MP_WORD_BITS;

   word borrow = 0;
   for(u32bit j = 0; j != y_size; j++)
      {
      dword temp = ((MP_RADIX + x[j]) - y[j]) - borrow;
      borrow = ((temp < MP_RADIX) ? 1 : 0);
      z[j] = (word)(temp - MP_RADIX);
      }
   for(u32bit j = y_size; j != x_size; j++)
      {
      if(borrow == 0)
         {
         memcpy(z + j, x + j, (x_size - j) * sizeof(word));
         break;
         }
      dword temp = (MP_RADIX + x[j]) - borrow;
      borrow = ((temp < MP_RADIX) ? 1 : 0);
      z[j] = (word)(temp - MP_RADIX);
      }
   }

}
